package com.eastrobot.auth.user.manager.controller;

package ..deTemp.1733059519722;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.eastrobot.arch.multitenant.context.TenantContext;
import com.eastrobot.arch.multitenant.utils.Base64Utils;
import com.eastrobot.auth.common.constants.AuthConstants;
import com.eastrobot.auth.common.context.CurrentUserContext;
import com.eastrobot.auth.common.entity.auth.AuthFeature;
import com.eastrobot.auth.common.entity.auth.AuthRoleXResource;
import com.eastrobot.auth.common.entity.auth.AuthUser;
import com.eastrobot.auth.common.entity.auth.ExtendApplication;
import com.eastrobot.auth.common.model.AppService;
import com.eastrobot.auth.common.model.ResponseEnum;
import com.eastrobot.auth.common.model.ResponseVO;
import com.eastrobot.auth.common.model.RoleInfo;
import com.eastrobot.auth.common.model.UserInfo;
import com.eastrobot.auth.common.model.param.LoginInfo;
import com.eastrobot.auth.common.model.param.TokenResp;
import com.eastrobot.auth.common.utils.RedisKeyManager;
import com.eastrobot.auth.sdk.model.SSOProperties;
import com.eastrobot.auth.sdk.service.MiddlePlatformService;
import com.eastrobot.auth.sdk.utils.JwtUtil;
import com.eastrobot.auth.sdk.utils.OpenApiSignUtil;
import com.eastrobot.auth.user.manager.controller.BaseController;
import com.eastrobot.auth.user.manager.model.AuthProperties;
import com.eastrobot.auth.user.manager.service.AppFeatureService;
import com.eastrobot.auth.user.manager.service.AuthManagerService;
import com.eastrobot.auth.user.manager.service.AuthUserService;
import com.eastrobot.auth.user.manager.service.ExtendAppService;
import com.eastrobot.auth.user.manager.service.FreezeService;
import com.eastrobot.auth.user.manager.service.LoginService;
import com.eastrobot.auth.user.manager.service.SysOperationLogService;
import com.eastrobot.auth.user.manager.utils.IpUtils;
import com.eastrobot.auth.user.manager.utils.MessageUtils;
import com.eastrobot.auth.user.manager.utils.SHAUtils;
import com.eastrobot.mp.db.service.SystemParameterService;
import com.eastrobot.mq.service.RedisService;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import java.io.InputStream;
import java.lang.invoke.SerializedLambda;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotBlank;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@Api(tags = {"鉴权管理"})
@RestController
@RequestMapping({"/auth"})
public class AuthController extends BaseController {
  private static final Logger log = LoggerFactory.getLogger(com.eastrobot.auth.user.manager.controller.AuthController.class);
  
  @Autowired
  private RedisService redisService;
  
  @Autowired
  private AuthUserService authUserService;
  
  @Autowired
  private AuthManagerService authManagerService;
  
  @Autowired
  private ExtendAppService extendAppService;
  
  @Autowired
  private SSOProperties ssoProperties;
  
  @Autowired
  private MiddlePlatformService middlePlatformService;
  
  @Autowired
  private LoginService loginService;
  
  @Autowired
  private AppFeatureService appFeatureService;
  
  @Autowired
  private SystemParameterService systemParameterService;
  
  @Autowired
  private AuthProperties authProperties;
  
  @Autowired
  private FreezeService freezeService;
  
  @Autowired
  private SysOperationLogService sysOperationLogService;
  
  @ApiOperation(value = "用户登录", notes = "输入用户名密码等信息登陆系统,Authentication可任意填写")
  @PostMapping({"login"})
  public ResponseVO login(@RequestBody LoginInfo loginInfo, HttpServletRequest request) {
    return loginTool(loginInfo, this.authProperties.isValidSign(), this.authProperties.isNeedPwd(), request);
  }
  
  @ApiOperation(value = "用户单点登录", notes = "输入用户名密码等信息登陆系统,Authentication可任意填写")
  @PostMapping({"loginSSO"})
  public ResponseVO loginSSO(@RequestBody LoginInfo loginInfo, HttpServletRequest request) {
    return loginTool(loginInfo, true, false, request);
  }
  
  private ResponseVO loginTool(LoginInfo loginInfo, boolean needSign, boolean needPassword, HttpServletRequest request) {
    String logPrefix = "";
    if ("SAAS".equals(this.platformMode))
      logPrefix = "租户: " + loginInfo.getTenantId() + " "; 
    String failLog = logPrefix + "用户: " + loginInfo.getUserName() + " 登录失败,原因: ";
    if (this.freezeService.userFreeze(loginInfo)) {
      Integer freezeTime = this.freezeService.userFreezeTime(loginInfo);
      log.info(failLog + ResponseEnum.ACCOUNT_BEEN_FROZEN.getMsg() + ", 冻结时间: " + freezeTime + "分钟");
      loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason0", new Object[0]) }), loginInfo.getUserName());
      return ResponseVO.error(ResponseEnum.ACCOUNT_BEEN_FROZEN, freezeTime);
    } 
    if ("SAAS".equals(this.platformMode)) {
      if (StringUtils.isBlank(loginInfo.getTenantId())) {
        log.info(failLog + ResponseEnum.MISSING_TENANT_ID.getMsg());
        loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason1", new Object[0]) }), loginInfo.getUserName());
        return ResponseVO.error(ResponseEnum.MISSING_TENANT_ID);
      } 
      TenantContext.setTenantId(loginInfo.getTenantId());
      if (TenantContext.getTenant() == null) {
        log.info(failLog + ResponseEnum.TENANT_DISABLED.getMsg());
        loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason2", new Object[0]) }), loginInfo.getUserName());
        return ResponseVO.error(ResponseEnum.TENANT_DISABLED);
      } 
      Date beginTime = TenantContext.getTenant().getBeginTime();
      Date endTime = TenantContext.getTenant().getEndTime();
      if ((beginTime != null && beginTime.after(new Date())) || (endTime != null && endTime.before(new Date()))) {
        log.info(failLog + ResponseEnum.TENANT_INVALID.getMsg());
        loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason3", new Object[0]) }), loginInfo.getUserName());
        return ResponseVO.error(ResponseEnum.TENANT_INVALID);
      } 
    } else {
      loginInfo.setTenantId(null);
    } 
    if (needSign) {
      String sign = request.getHeader("Ibn-Auth-Sign");
      if (StringUtils.isBlank(sign))
        sign = loginInfo.getSign(); 
      if (this.redisService.get(sign) != null) {
        log.warn(ResponseEnum.SIGN_USED.getMsg());
        loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason4", new Object[0]) }), loginInfo.getUserName());
        return ResponseVO.error(ResponseEnum.SIGN_USED);
      } 
      try {
        String genSign;
        Map<String, Object> paramsMap = new HashMap<>();
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(Long.parseLong(loginInfo.getTimestamp()));
        Calendar beforeTime = Calendar.getInstance();
        beforeTime.add(12, -30);
        Calendar afterTime = Calendar.getInstance();
        afterTime.add(12, 30);
        if (calendar.before(beforeTime) || calendar.after(afterTime)) {
          log.info(failLog + ResponseEnum.SIGN_ERROR.getMsg());
          loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason5", new Object[0]) }), loginInfo.getUserName());
          return ResponseVO.error(ResponseEnum.SIGN_ERROR);
        } 
        if (StringUtils.isBlank(sign)) {
          log.info(failLog + ResponseEnum.NEED_SIGN.getMsg());
          loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason6", new Object[0]) }), loginInfo.getUserName());
          return ResponseVO.error(ResponseEnum.NEED_SIGN);
        } 
        Map<String, Object> signMap = (new ObjectMapper()).<Map<String, Object>>convertValue(loginInfo, (TypeReference<Map<String, Object>>)new Object(this));
        for (String key : signMap.keySet()) {
          if (!"sign".equals(key) && 
            !"secret".equals(key) && 
            StringUtils.isNotBlank((String)signMap.get(key)))
            paramsMap.put(key, signMap.get(key)); 
        } 
        if (StringUtils.isNotBlank(loginInfo.getSecret())) {
          genSign = OpenApiSignUtil.genSign(paramsMap, Base64Utils.encoder(loginInfo.getSecret().substring(5, 20)));
        } else {
          genSign = OpenApiSignUtil.genSign(paramsMap, TenantContext.getTenant().getSecret());
        } 
        if (!sign.equals(genSign)) {
          log.info(failLog + ResponseEnum.SIGN_ERROR.getMsg());
          loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason5", new Object[0]) }), loginInfo.getUserName());
          return ResponseVO.error(ResponseEnum.SIGN_ERROR);
        } 
      } catch (Exception e) {
        log.error("", e);
        log.info(failLog + ResponseEnum.SIGN_ERROR.getMsg());
        loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason5", new Object[0]) }), loginInfo.getUserName());
        return ResponseVO.error(ResponseEnum.SIGN_ERROR);
      } 
      this.redisService.set(sign, Integer.valueOf(1), 1, TimeUnit.HOURS);
    } 
    QueryWrapper<AuthUser> authUserQueryWrapper = new QueryWrapper();
    authUserQueryWrapper.lambda().eq(AuthUser::getUserName, loginInfo.getUserName());
    String serviceName = StringUtils.isNotBlank(loginInfo.getServiceName()) ? loginInfo.getServiceName() : this.ssoProperties.getServiceName();
    Map<String, ExtendApplication> appMapFromCache = this.extendAppService.getExtendAppMapFromCache();
    if (!appMapFromCache.containsKey(serviceName)) {
      log.info(failLog + ResponseEnum.APP_UNAUTHORIZED.getMsg());
      loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason7", new Object[] { serviceName }) }), loginInfo.getUserName());
      return ResponseVO.error(ResponseEnum.APP_UNAUTHORIZED);
    } 
    ExtendApplication extendApplication = appMapFromCache.get(serviceName);
    if (extendApplication.getStartTime() != null && extendApplication.getStartTime().after(new Date())) {
      log.info(failLog + ResponseEnum.APP_AUTHORIZED_EXPIRE.getMsg());
      loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason8", new Object[] { serviceName }) }), loginInfo.getUserName());
      return ResponseVO.error(ResponseEnum.APP_AUTHORIZED_EXPIRE);
    } 
    if (extendApplication.getEndTime() != null && extendApplication.getEndTime().before(new Date())) {
      log.info(failLog + ResponseEnum.APP_AUTHORIZED_EXPIRE.getMsg());
      loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason8", new Object[] { serviceName }) }), loginInfo.getUserName());
      return ResponseVO.error(ResponseEnum.APP_AUTHORIZED_EXPIRE);
    } 
    List<AuthUser> authUsers = this.authUserService.list((Wrapper)authUserQueryWrapper);
    if (authUsers != null && authUsers.size() == 1) {
      AuthUser authUser = authUsers.get(0);
      AuthUser uu = new AuthUser();
      uu.setId(authUser.getId());
      String ipAddress = StringUtils.isBlank(loginInfo.getIp()) ? IpUtils.getIpAddr(request) : loginInfo.getIp();
      if (authUser.getStatus().intValue() == 1) {
        log.info(failLog + ResponseEnum.ACCOUNT_BEEN_FROZEN.getMsg());
        loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason0", new Object[0]) }), loginInfo.getUserName());
        return ResponseVO.error(ResponseEnum.ACCOUNT_BEEN_FROZEN);
      } 
      if (authUser.getEffectivePeriod() != null && authUser.getEffectivePeriod().after(new Date())) {
        log.info(failLog + ResponseEnum.USER_BEEN_INVALID.getMsg());
        loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason9", new Object[0]) }), loginInfo.getUserName());
        return ResponseVO.error(ResponseEnum.USER_BEEN_INVALID);
      } 
      if (authUser.getValidPeriod() != null && authUser.getValidPeriod().before(new Date())) {
        log.info(failLog + ResponseEnum.USER_BEEN_EXPIRED.getMsg());
        loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason10", new Object[0]) }), loginInfo.getUserName());
        return ResponseVO.error(ResponseEnum.USER_BEEN_EXPIRED);
      } 
      if (StringUtils.isNotBlank(authUser.getIpAddress()) && !authUser.getIpAddress().contains(ipAddress)) {
        log.info(failLog + ResponseEnum.IP_BEEN_FORBIDDEN.getMsg());
        loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason11", new Object[0]) }), loginInfo.getUserName());
        return ResponseVO.error(ResponseEnum.IP_BEEN_FORBIDDEN);
      } 
      String temporaryPasswordStr = (String)this.redisService.get(RedisKeyManager.keyTemporaryPassword(TenantContext.getTenantId(), loginInfo.getUserName()));
      String temporaryPassword = null;
      String encodePwd = null;
      try {
        if (StringUtils.isNotBlank(temporaryPasswordStr))
          temporaryPassword = SHAUtils.encodeSHA256Hex(temporaryPasswordStr); 
        if (StringUtils.isNotBlank(loginInfo.getUserPwd()))
          encodePwd = SHAUtils.encodeSHA256Hex(loginInfo.getUserPwd()); 
      } catch (Exception e) {
        log.error("", e);
      } 
      boolean enabled = this.systemParameterService.getBoolean("temporary.password.enabled", false);
      boolean useTemporaryPassword = (enabled && StringUtils.isNotBlank(temporaryPassword) && temporaryPassword.equals(loginInfo.getUserPwd()));
      if (!needPassword || authUser
        .getPassword().equals(loginInfo.getUserPwd()) || authUser
        .getPassword().equals(encodePwd) || useTemporaryPassword) {
        TokenResp tokenResp;
        long expire;
        if (authUser.getPassword().equals(loginInfo.getUserPwd()))
          uu.setPassword(encodePwd); 
        if (this.authManagerService.overOnlineNum(loginInfo.getUserName(), serviceName)) {
          log.info(failLog + ResponseEnum.OVER_ONLINE_NUM.getMsg());
          loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason12", new Object[0]) }), loginInfo.getUserName());
          return ResponseVO.error(ResponseEnum.OVER_ONLINE_NUM);
        } 
        this.authManagerService.kickOutUser(loginInfo.getUserName(), AuthConstants.KICK_OUT);
        Map<String, Object> map = new HashMap<>();
        map.put("userId", authUser.getId());
        map.put("userName", loginInfo.getUserName());
        map.put("tenantId", loginInfo.getTenantId());
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        map.put("loginTime", simpleDateFormat.format(new Date()));
        long tokenExpire = (this.systemParameterService.getInt("auth.user.token.expire", 1080) * 60);
        String token = JwtUtil.geneJsonWebToken(0L, map);
        if (this.authProperties.isDevEnabled()) {
          Calendar instance = Calendar.getInstance();
          instance.setTimeInMillis(System.currentTimeMillis());
          instance.add(1, 100);
          expire = instance.getTime().getTime();
        } else {
          expire = System.currentTimeMillis() + tokenExpire * 1000L;
        } 
        UserInfo userInfo = this.authManagerService.trans2UserInfo(token, authUsers.get(0), expire, loginInfo);
        this.authUserService.changeLoginTime(authUser.getId());
        String loginApp = authUser.getLoginApp();
        if (authUser.getFirstLogin() != null && authUser.getFirstLogin().booleanValue()) {
          List<String> roleIds = (List<String>)userInfo.getRoles().stream().map(RoleInfo::getId).collect(Collectors.toList());
          if (roleIds.contains("1")) {
            CurrentUserContext.setUser(userInfo);
            List<AuthFeature> authFeatures = this.appFeatureService.listEditFeature(roleIds, AuthRoleXResource.RESOURCE_TYPE_FEATURE, AuthRoleXResource.TYPE_INNER_JOIN, this.authAppName, null);
            if (authFeatures == null || authFeatures.isEmpty())
              this.authUserService.empowerAdminEx(); 
          } 
          uu.setFailNum(Integer.valueOf(0));
          this.redisService.delete(RedisKeyManager.keyFreezeNum(loginInfo.getTenantId(), loginInfo.getUserName()));
          tokenResp = new TokenResp(token, expire, true, false, authUser.getRealName(), loginInfo.getTenantId());
          if (!this.authProperties.isNeedChange()) {
            uu.setFirstLogin(Boolean.valueOf(false));
            tokenResp.setFirstLogin(false);
            userInfo.setFirstLogin(Boolean.valueOf(false));
            boolean firstLoginApp = false;
            if (StringUtils.isBlank(loginApp) || !loginApp.contains(serviceName)) {
              firstLoginApp = true;
              if (StringUtils.isBlank(loginApp)) {
                loginApp = serviceName;
              } else {
                loginApp = loginApp + "," + serviceName;
              } 
              uu.setLoginApp(loginApp);
            } 
            tokenResp.setFirstLoginApp(firstLoginApp);
          } 
          this.authUserService.updateById(uu);
        } else {
          boolean firstLoginApp = false;
          if (StringUtils.isBlank(loginApp) || !loginApp.contains(serviceName)) {
            firstLoginApp = true;
            if (StringUtils.isBlank(loginApp)) {
              loginApp = serviceName;
            } else {
              loginApp = loginApp + "," + serviceName;
            } 
            uu.setLoginApp(loginApp);
          } 
          uu.setFailNum(Integer.valueOf(0));
          this.authUserService.updateById(uu);
          this.redisService.delete(RedisKeyManager.keyFreezeNum(loginInfo.getTenantId(), loginInfo.getUserName()));
          tokenResp = new TokenResp(token, expire, (this.authProperties.isNeedPwd() && useTemporaryPassword), firstLoginApp, authUser.getRealName(), loginInfo.getTenantId());
        } 
        this.authManagerService.putInRedis(userInfo, token, new Date(expire));
        log.info(logPrefix + "用户: " + loginInfo.getUserName() + " 登录成功！");
        int passwordEffectiveDuration = this.systemParameterService.getInt("auth.password.effective.duration", 0);
        if (passwordEffectiveDuration > 0) {
          Calendar lastModifyCalendar = Calendar.getInstance();
          lastModifyCalendar.setTime(authUser.getLastModifyPasswordTime());
          lastModifyCalendar.add(6, passwordEffectiveDuration);
          Calendar nowDate = Calendar.getInstance();
          if (lastModifyCalendar.before(nowDate) && this.authProperties.isNeedChange())
            tokenResp.setFirstLogin(true); 
        } 
        this.sysOperationLogService.logOperation(AuthConstants.ModuleEnum.用户管理, AuthConstants.ActionEnum.登录, 
            
            MessageUtils.get("auth.opt.loginsuccess", new Object[0]), authUser
            .getId(), authUser.getRealName());
        return ResponseVO.success(tokenResp);
      } 
      int failNum = authUser.getFailNum().intValue() + 1;
      int failNumConf = this.systemParameterService.getInt("auth.login.fail.num");
      if (failNum >= failNumConf) {
        uu.setStatus(AuthUser.FREEZE);
        this.freezeService.freezeUser(loginInfo);
      } 
      uu.setFailNum(Integer.valueOf(failNum));
      this.authUserService.updateById(uu);
      log.info(failLog + ResponseEnum.USER_AUTHENTICATION_FAILED.getMsg());
      loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason13", new Object[0]) }), loginInfo.getUserName());
      return ResponseVO.error(ResponseEnum.USER_AUTHENTICATION_FAILED);
    } 
    log.info(failLog + ResponseEnum.USER_AUTHENTICATION_FAILED.getMsg());
    loginFail(MessageUtils.get("auth.opt.loginfail", new Object[] { loginInfo.getUserName(), MessageUtils.get("auth.opt.reason13", new Object[0]) }), loginInfo.getUserName());
    return ResponseVO.error(ResponseEnum.USER_AUTHENTICATION_FAILED);
  }
  
  @ApiOperation(value = "单点登录", notes = "通过token获取用户信息,token校验失败时则跳转登录页")
  @PostMapping({"ssoLogin"})
  public ResponseVO userInfo(@RequestBody AppService param, @NotBlank @RequestHeader String Authorization) {
    return ResponseVO.success(this.middlePlatformService.getUserInfo(Authorization, param));
  }
  
  @ApiOperation(value = "登出", notes = "用户登出")
  @GetMapping({"loginOut"})
  public ResponseVO loginOut(@ApiParam(name = "Authorization", type = "String", value = "token") @RequestHeader String Authorization, @ApiParam(name = "loginOutReason", type = "String", value = "loginOutReason") @RequestParam(required = false) String loginOutReason) {
    String tenantId = null;
    if ("SAAS".equals(this.platformMode))
      tenantId = TenantContext.getTenantId(); 
    this.loginService.logout(Authorization, loginOutReason);
    if ("SAAS".equals(this.platformMode))
      return ResponseVO.success(tenantId); 
    return ResponseVO.success();
  }
  
  @GetMapping({"validLicense"})
  public ResponseVO validLicense(@RequestParam String serviceName, HttpServletRequest request) {
    String ipAddr = IpUtils.getIpAddr(request);
    serviceName = StringUtils.isNotBlank(serviceName) ? serviceName : this.ssoProperties.getServiceName();
    Object o = this.redisService.get(RedisKeyManager.keyValidLicenseToken(serviceName, ipAddr));
    if (o == null) {
      Map<String, ExtendApplication> appMapFromCache = this.extendAppService.getExtendAppMapFromCache();
      if (!appMapFromCache.containsKey(serviceName)) {
        o = ResponseEnum.APP_UNAUTHORIZED.name();
      } else {
        ExtendApplication extendApplication = appMapFromCache.get(serviceName);
        if (extendApplication.getStartTime() != null && extendApplication.getStartTime().after(new Date())) {
          o = ResponseEnum.APP_AUTHORIZED_EXPIRE.name();
        } else if (extendApplication.getEndTime() != null && extendApplication.getEndTime().before(new Date())) {
          o = ResponseEnum.APP_AUTHORIZED_EXPIRE.name();
        } else {
          o = "success";
        } 
      } 
      this.redisService.set(RedisKeyManager.keyValidLicenseToken(serviceName, ipAddr), o, 1, TimeUnit.SECONDS);
    } 
    if ("success".equals(o))
      return ResponseVO.success(); 
    return ResponseVO.error(ResponseEnum.valueOf(o.toString()));
  }
  
  private static String versionStr = null;
  
  @GetMapping({"version"})
  public String getVersion() {
    if (StringUtils.isNotBlank(versionStr))
      return versionStr; 
    InputStream inputStream = null;
    try {
      ClassPathResource resource = new ClassPathResource("version.txt");
      inputStream = resource.getInputStream();
      List<String> versionList = IOUtils.readLines(inputStream, Charset.defaultCharset());
      versionStr = versionList.get(0);
      return versionStr;
    } catch (Exception e) {
      log.error("", e);
    } finally {
      if (inputStream != null)
        try {
          inputStream.close();
        } catch (Exception exception) {} 
    } 
    return "版本查询失败";
  }
  
  private void loginFail(String reason, String userName) {
    QueryWrapper<AuthUser> selectWrap = new QueryWrapper();
    selectWrap.lambda().eq(AuthUser::getUserName, userName);
    List<AuthUser> list = this.authUserService.list((Wrapper)selectWrap);
    if (list != null && list.size() == 1) {
      this.sysOperationLogService.logOperation(AuthConstants.ModuleEnum.用户管理, AuthConstants.ActionEnum.登录, reason, ((AuthUser)list.get(0)).getId(), ((AuthUser)list.get(0)).getRealName());
    } else {
      this.sysOperationLogService.logOperation(AuthConstants.ModuleEnum.用户管理, AuthConstants.ActionEnum.登录, reason, null, userName);
    } 
  }
}
